@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2020 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -20,9 +20,6 @@
 #include <ags/X/ags_plugin_browser.h>
 #include <ags/X/ags_plugin_browser_callbacks.h>
 
-#include <ags/libags.h>
-#include <ags/libags-audio.h>
-
 #include <ags/X/ags_lv2_browser.h>
 #include <ags/X/ags_dssi_browser.h>
 #include <ags/X/ags_ladspa_browser.h>
@@ -147,7 +144,7 @@ ags_plugin_browser_init(AgsPluginBrowser *plugin_browser)
   plugin_browser->flags = 0;
 
   vbox = (GtkVBox *) gtk_vbox_new(FALSE, 0);
-  gtk_box_pack_start((GtkBox *) plugin_browser->dialog.vbox,
+  gtk_box_pack_start((GtkBox *) gtk_dialog_get_content_area(GTK_DIALOG(plugin_browser)),
 		     GTK_WIDGET(vbox),
 		     FALSE, FALSE,
 		     0);
@@ -307,7 +304,7 @@ ags_plugin_browser_reset(AgsApplicable *applicable)
  *
  * Returns: the active plugin filename
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 gchar*
 ags_plugin_browser_get_plugin_filename(AgsPluginBrowser *plugin_browser)
@@ -329,7 +326,7 @@ ags_plugin_browser_get_plugin_filename(AgsPluginBrowser *plugin_browser)
  *
  * Returns: the active plugin effect
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 gchar*
 ags_plugin_browser_get_plugin_effect(AgsPluginBrowser *plugin_browser)
@@ -351,7 +348,7 @@ ags_plugin_browser_get_plugin_effect(AgsPluginBrowser *plugin_browser)
  *
  * Returns: the new #AgsPluginBrowser
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 AgsPluginBrowser*
 ags_plugin_browser_new(GtkWidget *parent_window)
